function [Z,E,W] = MLRR(X,SS,para,rho,DEBUG)
% This matlab code implements linearized ADM for graph regularized LRR
% problem
% ----------------------------------------------
% min |Z|_*+lambda*|E|_2,1+alpha*|W|_1+beta*g(W)
% s.t., X = XZ+E, Z=W, W>=0
% ----------------------------------------------
% inputs:
%        X -- D*N data matrix
%        SS -- coefficient matrix learned by sparse manifold fitting
%        para -- lambda,alpha,beta are regularization parameters
% outputs:
%        Z -- N*N representation matrix
%        E -- D*N sparse error matrix
%        W -- auxiliary variable
%
% written by Yong Peng, University of Michigan, Ann Arbor
% August-2013, stany.peng@gmail.com
% 
clear global;

% addpath PROPACK;

if ~exist('DEBUG','var')
    DEBUG = 0;
end

lambda = para.lambda;
alpha  = para.alpha;
beta   = para.beta;

normfX = norm(X,'fro');
tol1   = 1e-8; % threshold for the error in the constaint
tol2   = 1e-6; % threshold for the change in the solutions
[d, n] = size(X);
% opt.tol = tol2; % precision for computing the partial SVD
% opt.p0  = ones(n,1);

maxIter = 1000;

max_mu  = 1e10;
norm2X  = norm(X,2);
% mu      = min(d,n)*tol2;
mu = 0.1;

% eta needs to be larger than ||X||_2^2, but need not to be too large
eta     = norm2X*norm2X;

%% Initialize optimization variables
Z = zeros(n,n);
W = zeros(n,n);
E = sparse(d,n);

Y1 = zeros(d,n);
Y2 = zeros(n,n);

XZ = zeros(d,n); % XZ = X*Z;

sv = 5;
svp = sv;

%% Start main loop
iter = 0;

if DEBUG
    disp(['initial, rank(Z)=' num2str(rank(Z))]);
end

G = (eye(n)-SS)*(eye(n)-SS)';

while iter < maxIter
    % disp(['iter=' num2str(iter)]);
    
    iter = iter + 1;
    
    % copy Z, W and E to compute the change in solutions
    Zk = Z;
    Wk = W;
    Ek = E;
    
    % update 'W': soft-thresholding (shrinkage) opearator
    a = norm(2*beta*G+mu*eye(n),'fro');
    temp = mu*(Z+Y2/mu)/a;
    W = max(0,temp-alpha/a) + min(0, temp+alpha/a);
    % temp = mu*(Z+Y2/mu);
    % W = max(0,temp-alpha/mu) + min(0, temp+alpha/mu);
    W = max(W,0);
      
    % update 'Z': singular value thresholding operator   
    M = Z + (X'*(X-XZ-E+Y1/mu) - (Z-W+Y2/mu))/eta;
    % [U,S,V] = lansvd(M,n,n,sv,'L',opt); % using 'PROPACK' package
    [U,S,V] = svd(M,'econ');
    S = diag(S);
    svp = length(find(S > 1/(mu*eta)));
    if svp < sv
        sv = min(svp + 1, n);
    else
        sv = min(svp + round(0.05*n), n);
    end
    if svp >= 1
        S = S(1:svp) - 1/(mu*eta);
    else
        svp = 1;
        S = 0;
    end
    A.U = U(:,1:svp);
    A.s = S;
    A.V = V(:,1:svp);
    Z = A.U*diag(A.s)*A.V';
    
    
    % update 'E': L21 norm shrinkage operator
    E = solve_l1l2(X-XZ+Y1/mu,lambda/mu);
    
    
    % convergence judgement and parameters update
    relChgZ = norm(Z-Zk, 'fro') / normfX;
    relChgW = norm(W-Wk, 'fro') / normfX;
    relChgE = norm(E-Ek, 'fro') / normfX;
    relChg  = max(max(relChgZ, relChgW), relChgE);
    
    % introducing XZ to avoid computing X*Z multiple times, 
    % which has O(n^3) complexity.
    XZ = X*Z;
    dY1 = X - XZ - E;
    recErr = norm(dY1, 'fro')/normfX;
    
    convergenced = (recErr < tol1) && (mu*relChg < tol2);
    
    if DEBUG
        if iter == 1 || mod(iter,50)==0 || convergenced
            disp(['iter '     num2str(iter) ',mu='     num2str(mu,'%2.1e') ...
                  ',rank(Z)=' num2str(rank(Z,1e-3*norm(Z,2)))  ',relChg=' num2str(relChg,'%2.3e') ...
                  ',recErr='   num2str(recErr)]);
        end
    end
    
    if convergenced
        disp('converged!!');
        % if recErr < tol1 && mu*relChg < tol2 % this is the correct
        % stopping criteria
        break;
    else
        Y1 = Y1 + mu*dY1;
        Y2 = Y2 + mu*(Z-W);
        
        if mu*relChg < tol2
            mu = min(max_mu, mu*rho);
        end
    end
end

function E = solve_l1l2(W,lambda)
n = size(W,2);
E = W;
for i = 1:n
    E(:,i) = solve_l2(W(:,i),lambda);
end
    
function x = solve_l2(w,lambda)
% min lambda |x|_2+|x-w|_2^2
nw = norm(w);
if nw > lambda
    x = (nw-lambda)*w/nw;
else
    x = zeros(length(w),1);
end
